@@ -56,7 +56,7 @@ def build_parser():
     beta_bake_parser = subparsers.add_parser('beta_bake', help='All encompasing pipeline that will find GEO datasets in any form, download, and convert into a pickled dataframe of beta-values. Just specify the GEO_ID.')
     beta_bake_parser.set_defaults(func=cli_beta_bakery)
 
-    download_parser = subparsers.add_parser('download', help='Downloads the specified series from GEO or ArrayExpress.')
+    download_parser = subparsers.add_parser('download', help='Downloads the specified series from GEO or ArrayExpress, if IDATs are available.')
     download_parser.set_defaults(func=cli_download)
 
     meta_parser = subparsers.add_parser('meta_data', help='Creates a meta_data dataframe from GEO MINiML XML file. Specify the GEO id.')
@@ -129,7 +129,7 @@ def cli_process(cmd_args):
     parser.add_argument(
         '-n', '--sample_name',
         required=False,
-        nargs='*', # -- this flag support making a list of of each -n
+        nargs='+', # -- this flag support making a list of each -n
         help='Sample(s) to process. You can pass multiple sample names like this: `python -m methylprep process -d . --all --no_sample_sheet -n Sample_1 Sample_2 Sample_3`',
     )
 
@@ -204,6 +204,14 @@ def cli_process(cmd_args):
         help='By default, any beta-values or m-values output will contain all probes. If True, those probes that fail the p-value signal:noise detection are replaced with NaNs in dataframes in beta_values and m_value output.'
     )
 
+    parser.add_argument(
+        '--pneg_ecdf',
+        required=False,
+        action='store_true',
+        default=False,
+        help='Calculates a pvalue for each probe against ECDF of negative control intensity and outputs as pNegECDF_values.pkl if export_poobah option specified. Scores also included in csv output.'
+    )
+
     parser.add_argument(
         '--export_poobah',
         required=False,
@@ -212,6 +220,13 @@ def cli_process(cmd_args):
         help='If specified, exports a pickled dataframe of the poobah p-values per sample.'
     )
 
+    parser.add_argument(
+        '-f', '--file_format',
+        required=False,
+        default='pickle',
+        help='Specify `parquet` instead of default `pickle`'
+    )
+
     parser.add_argument(
         '--minfi',
         required=False,
@@ -276,8 +291,10 @@ def cli_process(cmd_args):
         poobah=args.poobah,
         export_poobah=args.export_poobah,
         quality_mask=(not args.no_quality_mask),
-        sesame=(not args.minfi) # default 'sesame' method can be turned off using --minfi
-        )
+        sesame=(not args.minfi), # default 'sesame' method can be turned off using --minfi,
+        pneg_ecdf=args.pneg_ecdf,
+        file_format=args.file_format
+    )
 
 
 def cli_beta_bakery(cmd_args):
@@ -353,6 +370,7 @@ def cli_beta_bakery(cmd_args):
 
     args = parser.parse_args(cmd_args)
     args.project_name = args.id
+    args.move = True # moves all files out of temp-working-folder if called via CLI
     delattr(args,'id')
     if args.no_clean == True:
         args.clean = False
@@ -636,8 +654,16 @@ def cli_sample_sheet(cmd_args):
 def cli_alert(cmd_args):
     parser = DefaultParser(
         prog='methylprep alert',
-        description="Regularly search GEO for new data, filtered by keyword, and updating only if new data found."
-        )
+        description="""Searches GEO for datasets, filtered by keyword, and saves results to <keyword-pattern>_meta.csv.""",
+    )
+    parser.add_argument(
+        '-v', '--verbose',
+        required=False,
+        help="""Verbose reports to page and saves csv.""",
+        type=str,
+        default=''
+    )
+
     parser.add_argument(
         '-k', '--keyword',
         required=False,
