@@ -4,6 +4,7 @@ import pandas as pd
 import numpy as np
 import struct
 from pprint import pprint
+import logging
 
 # App
 from ..utils import (
@@ -19,6 +20,8 @@ from ..utils import (
     npread,
 )
 
+LOGGER = logging.getLogger(__name__)
+LOGGER.setLevel( logging.WARNING )
 
 __all__ = ['IdatDataset']
 
@@ -140,6 +143,7 @@ class IdatDataset():
         bit='float32',
     ):
         """Initializes the IdatDataset, reads and parses the IDAT file."""
+        self.verbose = verbose
         self.channel = channel
         self.barcode = None
         self.chip_type = None
@@ -160,7 +164,9 @@ class IdatDataset():
                 raise ValueError('Not a version 3 IDAT file. Unsupported IDAT version.')
 
             self.probe_means = self.read(idat_file)
-            if verbose:
+            if self.overflow_check() is False:
+                LOGGER.warning("IDAT: contains negative probe values (uint16 overflow error)")
+            if self.verbose:
                 self.meta(idat_file)
 
     @staticmethod
@@ -245,13 +251,13 @@ class IdatDataset():
         self.n_snps_read = read_int(idat_file)
 
         seek_to_section(IdatSectionCode.NUM_BEADS)
-        self.n_beads = npread(idat_file, '<u1', self.n_snps_read)
+        self.n_beads = npread(idat_file, '<u1', self.n_snps_read) # was <u1
 
         seek_to_section(IdatSectionCode.ILLUMINA_ID)
         illumina_ids = npread(idat_file, '<i4', self.n_snps_read)
 
         seek_to_section(IdatSectionCode.MEAN)
-        probe_means = npread(idat_file, '<u2', self.n_snps_read)
+        probe_means = npread(idat_file, '<u2', self.n_snps_read) # '<u2' reads data as numpy unsigned-float16
 
         seek_to_section(IdatSectionCode.RUN_INFO)
         runinfo_entry_count, = struct.unpack('<L', idat_file.read(4))
@@ -266,39 +272,41 @@ class IdatDataset():
         if self.include_std_dev and self.include_n_beads:
             seek_to_section(IdatSectionCode.STD_DEV)
             std_devs = npread(idat_file, '<u2', self.n_snps_read)
-            print(len(probe_means), len(std_devs), len(self.n_beads))
+            # print(len(probe_means), len(std_devs), len(self.n_beads))
             data_frame = pd.DataFrame(
                 data={'mean_value':probe_means, 'std_dev':std_devs, 'n_beads':self.n_beads},
                 index=illumina_ids,
                 columns=['mean_value','std_dev','n_beads'],
-                dtype='float32', # int16 could work, and reduce memory by 1/2, but some raw values were > 32127 -- without prenormalization, you get negative values back, which breaks stuff.
+                dtype=self.bit, # int16 could work, and reduce memory by 1/2, but some raw values were > 32127 -- without prenormalization, you get negative values back, which breaks stuff.
             )
+
         elif self.include_std_dev:
             seek_to_section(IdatSectionCode.STD_DEV)
             std_devs = npread(idat_file, '<u2', self.n_snps_read)
-            print(len(probe_means), len(std_devs))
+            # print(len(probe_means), len(std_devs))
             data_frame = pd.DataFrame(
                 data={'mean_value':probe_means, 'std_dev':std_devs},
                 index=illumina_ids,
                 columns=['mean_value','std_dev'],
-                dtype='float32', # int16 could work, and reduce memory by 1/2, but some raw values were > 32127 -- without prenormalization, you get negative values back, which breaks stuff.
+                dtype=self.bit, # int16 could work, and reduce memory by 1/2, but some raw values were > 32127 -- without prenormalization, you get negative values back, which breaks stuff.
             )
         elif self.include_n_beads:
             data_frame = pd.DataFrame(
                 data={'mean_value':probe_means, 'n_beads':self.n_beads},
                 index=illumina_ids,
                 columns=['mean_value','n_beads'],
-                dtype='float32', # int16 could work, and reduce memory by 1/2, but some raw values were > 32127 -- without prenormalization, you get negative values back, which breaks stuff.
+                dtype=self.bit, # int16 could work, and reduce memory by 1/2, but some raw values were > 32127 -- without prenormalization, you get negative values back, which breaks stuff.
             )
         else:
-            probe_records = dict(zip(illumina_ids, probe_means))
+            # casting astype(int) fixes pandas bug reading uint16, by coercing to float32-compatible format.
+            probe_records = dict(zip(illumina_ids, probe_means.astype(int)))
             data_frame = pd.DataFrame.from_dict(
                 data=probe_records,
                 orient='index',
                 columns=['mean_value'],
-                dtype='float32', # int16 could work, and reduce memory by 1/2, but some raw values were > 32127 -- without prenormalization, you get negative values back, which breaks stuff.
+                dtype=self.bit, # int16 could work, and reduce memory by 1/2, but some raw values were > 32127 -- without prenormalization, you get negative values back, which breaks stuff.
             )
-        data_frame.index.name = 'illumina_id'
+            data_frame.index.name = 'illumina_id'
 
         if self.bit == 'float16':
             data_frame = data_frame.clip(upper=32127)
@@ -341,3 +349,10 @@ class IdatDataset():
             if line[1] == 'Scan':
                 print(f"300 run info: {line}")
                 break
+
+    def overflow_check(self):
+        if hasattr(self, 'probe_means'):
+            if (self.probe_means.values < 0).any():
+                # n_affected = self.probe_means[self.probe_means.mean_value < 0].count().values[0]
+                return False
+        return True # passes, no misread probes
