@@ -10,7 +10,7 @@ from ..models import (
 from ..models.probes import FG_PROBE_SUBSETS
 from ..files import IdatDataset
 from ..utils import inner_join_data
-from ..utils.progress_bar import * # checks environment and imports tqdm appropriately.
+# from ..utils.progress_bar import * # checks environment and imports tqdm appropriately.
 from collections import Counter
 
 __all__ = ['RawDataset', 'MethylationDataset', 'get_raw_datasets', 'get_array_type']
@@ -82,10 +82,10 @@ def get_raw_datasets(sample_sheet, sample_name=None, from_s3=None, meta_only=Fal
     elif from_s3 and not meta_only:
         parser = RawDataset.from_sample_s3
         zip_reader = from_s3
-        raw_datasets = tqdm([parser(zip_reader, sample) for sample in samples], total=len(samples), desc='Getting raw datasets')
+        raw_datasets = [parser(zip_reader, sample) for sample in samples]
     elif not from_s3 and not meta_only:
         parser = RawDataset.from_sample
-        raw_datasets = tqdm([parser(sample) for sample in samples], total=len(samples), desc='Getting raw datasets')
+        raw_datasets = [parser(sample) for sample in samples]
 
     if not meta_only:
         # ensure all idat files have same number of probes
@@ -99,7 +99,7 @@ def get_raw_datasets(sample_sheet, sample_name=None, from_s3=None, meta_only=Fal
             probe_sample_counts = Counter([dataset.n_snps_read for dataset in raw_datasets])
             samples_by_probe_count = {probe_count:[] for probe_count in list(probe_counts)}
             for dataset in raw_datasets:
-                sample_name = f"{dataset.sample.sentrix_id}_{dataset.sample.sentrix_id}"
+                sample_name = f"{dataset.sample.sentrix_id}_{dataset.sample.sentrix_position}"
                 samples_by_probe_count[dataset.n_snps_read].append(sample_name)
             LOGGER.error(f'Samples grouped by probe count: {probe_sample_counts.most_common()}')
             LOGGER.error(f'{samples_by_probe_count}')
