@@ -29,7 +29,7 @@ from .postprocess import (
 )
 from ..utils import ensure_directory_exists, is_file_like
 from .preprocess import preprocess_noob, _apply_sesame_quality_mask
-from .p_value_probe_detection import _pval_sesame_preprocess
+from .p_value_probe_detection import _pval_sesame_preprocess, _pval_neg_ecdf
 from .infer_channel_switch import infer_type_I_probes
 from .dye_bias import nonlinear_dye_bias_correction
 from .multi_array_idat_batches import check_array_folders
@@ -46,7 +46,7 @@ def run_pipeline(data_dir, array_type=None, export=False, manifest_filepath=None
                  save_uncorrected=False, save_control=True, meta_data_frame=True,
                  bit='float32', poobah=False, export_poobah=False,
                  poobah_decimals=3, poobah_sig=0.05, low_memory=True,
-                 sesame=True, quality_mask=None, **kwargs):
+                 sesame=True, quality_mask=None, pneg_ecdf=False, file_format='pickle', **kwargs):
     """The main CLI processing pipeline. This does every processing step and returns a data set.
 
     Required Arguments:
@@ -96,6 +96,10 @@ def run_pipeline(data_dir, array_type=None, export=False, manifest_filepath=None
             if True, saves a file, "sample_sheet_meta_data.pkl" with samplesheet info.
         export [default: False]
             if True, exports a CSV of the processed data for each idat file in sample.
+        file_format [default: pickle; optional: parquet]
+            Matrix style files are faster to load and process than CSVs, and python supports two
+            types of binary formats: pickle and parquet. Parquet is readable by other languages,
+            so it is an option starting v1.7.0.
         save_uncorrected [default: False]
             if True, adds two additional columns to the processed.csv per sample (meth and unmeth),
             representing the raw fluorescence intensities for all probes.
@@ -217,7 +221,13 @@ def run_pipeline(data_dir, array_type=None, export=False, manifest_filepath=None
             # mouse is determined by the array_type match, but you can suppress creating this file here
             do_mouse = True if 'mouse' in pipeline_exports else False
             save_control = True if 'control' in pipeline_exports else False
-
+    if file_format == 'parquet':
+        try:
+            pd.DataFrame().to_parquet()
+        except AttributeError():
+            LOGGER.error("parquet is not installed in your environment; reverting to pickle format")
+            file_format = 'pickle'
+    suffix = 'parquet' if file_format == 'parquet' else 'pkl'
 
     LOGGER.info('Running pipeline in: %s', data_dir)
     if bit not in ('float64','float32','float16'):
@@ -252,7 +262,7 @@ def run_pipeline(data_dir, array_type=None, export=False, manifest_filepath=None
         if not isinstance(sample_name,(list,tuple)):
             raise SystemExit(f"sample_name must be a list of sample_names")
         matched_samples = [sample.name for sample in samples if sample.name in sample_name]
-        if matched_samples != sample_name:
+        if set(matched_samples) != set(sample_name):
             possible_sample_names = [sample.name for sample in samples]
             unmatched_samples = [_sample for _sample in sample_name if _sample not in possible_sample_names]
             raise SystemExit(f"Your sample_name filter does not match the samplesheet; these samples were not found: {unmatched_samples}")
@@ -308,7 +318,7 @@ def run_pipeline(data_dir, array_type=None, export=False, manifest_filepath=None
     missing_probe_errors = {'noob': [], 'raw':[]}
 
     for batch_num, batch in enumerate(batches, 1):
-        idat_datasets = parse_sample_sheet_into_idat_datasets(sample_sheet, sample_name=batch, from_s3=None, meta_only=False) # replaces get_raw_datasets
+        idat_datasets = parse_sample_sheet_into_idat_datasets(sample_sheet, sample_name=batch, from_s3=None, meta_only=False, bit=bit) # replaces get_raw_datasets
         # idat_datasets are a list; each item is a dict of {'green_idat': ..., 'red_idat':..., 'array_type', 'sample'} to feed into SigSet
         #--- pre v1.5 --- raw_datasets = get_raw_datasets(sample_sheet, sample_name=batch)
         if array_type is None: # use must provide either the array_type or manifest_filepath.
@@ -332,11 +342,14 @@ def run_pipeline(data_dir, array_type=None, export=False, manifest_filepath=None
                 do_nonlinear_dye_bias=do_nonlinear_dye_bias, # start of run_pipeline sets this to True, False, or None
                 debug=kwargs.get('debug',False),
                 sesame=sesame,
+                pneg_ecdf=pneg_ecdf,
+                file_format=file_format,
             )
             data_container.process_all()
 
-            if export: # as CSV
-                output_path = data_container.sample.get_export_filepath()
+            if export: # as CSV or parquet
+                suffix = 'parquet' if file_format == 'parquet' else 'csv'
+                output_path = data_container.sample.get_export_filepath(extension=suffix)
                 data_container.export(output_path)
                 export_paths.add(output_path)
                 # this tidies-up the tqdm by moving errors to end of batch warning.
@@ -374,110 +387,68 @@ def run_pipeline(data_dir, array_type=None, export=False, manifest_filepath=None
 
         if kwargs.get('debug'): LOGGER.info('[finished SampleDataContainer processing]')
 
+        def _prepare_save_out_file(df, file_stem, uint16=False):
+            out_name = f"{file_stem}_{batch_num}" if batch_size else file_stem
+            if uint16 and file_format != 'parquet':
+                df = df.astype('float32') if df.isna().sum().sum() > 0 else df.astype('uint16')
+            else:
+                df = df.astype('float32')
+            if df.shape[1] > df.shape[0]:
+                df = df.transpose() # put probes as columns for faster loading.
+            # sort sample names
+            df = df.sort_index().reindex(sorted(df.columns), axis=1)
+            if file_format == 'parquet':
+                # put probes in rows; format is optimized for same-type storage so it won't really matter
+                df.to_parquet(Path(data_dir,f"{out_name}.parquet"))
+            else:
+                df.to_pickle(Path(data_dir, f"{out_name}.pkl"))
+            LOGGER.info(f"saved {out_name}")
+
         if betas:
             df = consolidate_values_for_sheet(batch_data_containers, postprocess_func_colname='beta_value', bit=bit, poobah=poobah, exclude_rs=True)
-            if not batch_size:
-                pkl_name = 'beta_values.pkl'
-            else:
-                pkl_name = f'beta_values_{batch_num}.pkl'
-            if df.shape[1] > df.shape[0]:
-                df = df.transpose() # put probes as columns for faster loading.
-            df = df.astype('float32')
-            df = df.sort_index().reindex(sorted(df.columns), axis=1)
-            pd.to_pickle(df, Path(data_dir,pkl_name))
-            LOGGER.info(f"saved {pkl_name}")
+            _prepare_save_out_file(df, 'beta_values')
         if m_value:
             df = consolidate_values_for_sheet(batch_data_containers, postprocess_func_colname='m_value', bit=bit, poobah=poobah, exclude_rs=True)
-            if not batch_size:
-                pkl_name = 'm_values.pkl'
-            else:
-                pkl_name = f'm_values_{batch_num}.pkl'
-            if df.shape[1] > df.shape[0]:
-                df = df.transpose() # put probes as columns for faster loading.
-            df = df.astype('float32')
-            df = df.sort_index().reindex(sorted(df.columns), axis=1)
-            pd.to_pickle(df, Path(data_dir,pkl_name))
-            LOGGER.info(f"saved {pkl_name}")
+            _prepare_save_out_file(df, 'm_values')
         if (do_save_noob is not False) or betas or m_value:
             df = consolidate_values_for_sheet(batch_data_containers, postprocess_func_colname='noob_meth', bit=bit, poobah=poobah, exclude_rs=True)
-            if not batch_size:
-                pkl_name = 'noob_meth_values.pkl'
-            else:
-                pkl_name = f'noob_meth_values_{batch_num}.pkl'
-            if df.shape[1] > df.shape[0]:
-                df = df.transpose() # put probes as columns for faster loading.
-            df = df.astype('float32') if df.isna().sum().sum() > 0 else df.astype('uint16')
-            df = df.sort_index().reindex(sorted(df.columns), axis=1)
-            pd.to_pickle(df, Path(data_dir,pkl_name))
-            LOGGER.info(f"saved {pkl_name}")
-            # TWO PARTS
+            _prepare_save_out_file(df, 'noob_meth_values', uint16=True)
             df = consolidate_values_for_sheet(batch_data_containers, postprocess_func_colname='noob_unmeth', bit=bit, poobah=poobah, exclude_rs=True)
-            if not batch_size:
-                pkl_name = 'noob_unmeth_values.pkl'
-            else:
-                pkl_name = f'noob_unmeth_values_{batch_num}.pkl'
-            if df.shape[1] > df.shape[0]:
-                df = df.transpose() # put probes as columns for faster loading.
-            df = df.astype('float32') if df.isna().sum().sum() > 0 else df.astype('uint16')
-            df = df.sort_index().reindex(sorted(df.columns), axis=1)
-            pd.to_pickle(df, Path(data_dir,pkl_name))
-            LOGGER.info(f"saved {pkl_name}")
-
-        #if (betas or m_value) and save_uncorrected:
+            _prepare_save_out_file(df, 'noob_unmeth_values', uint16=True)
         if save_uncorrected:
             df = consolidate_values_for_sheet(batch_data_containers, postprocess_func_colname='meth', bit=bit, poobah=False, exclude_rs=True)
-            if not batch_size:
-                pkl_name = 'meth_values.pkl'
-            else:
-                pkl_name = f'meth_values_{batch_num}.pkl'
-            if df.shape[1] > df.shape[0]:
-                df = df.transpose() # put probes as columns for faster loading.
-            df = df.astype('float32') if df.isna().sum().sum() > 0 else df.astype('uint16')
-            df = df.sort_index().reindex(sorted(df.columns), axis=1)
-            pd.to_pickle(df, Path(data_dir,pkl_name))
-            LOGGER.info(f"saved {pkl_name}")
-            # TWO PARTS
+            _prepare_save_out_file(df, 'meth_values', uint16=True)
             df = consolidate_values_for_sheet(batch_data_containers, postprocess_func_colname='unmeth', bit=bit, poobah=False, exclude_rs=True)
-            if not batch_size:
-                pkl_name = 'unmeth_values.pkl'
-            else:
-                pkl_name = f'unmeth_values_{batch_num}.pkl'
-            if df.shape[1] > df.shape[0]:
-                df = df.transpose() # put probes as columns for faster loading.
-            df = df.astype('float32') if df.isna().sum().sum() > 0 else df.astype('uint16')
-            df = df.sort_index().reindex(sorted(df.columns), axis=1)
-            pd.to_pickle(df, Path(data_dir,pkl_name))
-            LOGGER.info(f"saved {pkl_name}")
+            _prepare_save_out_file(df, 'unmeth_values', uint16=True)
 
         if manifest.array_type == ArrayType.ILLUMINA_MOUSE and do_mouse:
             # save mouse specific probes
             if not batch_size:
-                mouse_probe_filename = f'mouse_probes.pkl'
+                mouse_probe_filename = f'mouse_probes.{suffix}'
             else:
-                mouse_probe_filename = f'mouse_probes_{batch_num}.pkl'
-            consolidate_mouse_probes(batch_data_containers, Path(data_dir, mouse_probe_filename))
+                mouse_probe_filename = f'mouse_probes_{batch_num}.{suffix}'
+            consolidate_mouse_probes(batch_data_containers, Path(data_dir, mouse_probe_filename), file_format)
             LOGGER.info(f"saved {mouse_probe_filename}")
 
         if export:
             export_path_parents = list(set([str(Path(e).parent) for e in export_paths]))
-            LOGGER.info(f"[!] Exported results (csv) to: {export_path_parents}")
+            LOGGER.info(f"[!] Exported results ({file_format}) to: {export_path_parents}")
 
         if export_poobah:
             if all(['poobah_pval' in e._SampleDataContainer__data_frame.columns for e in batch_data_containers]):
-                # this option will save a pickled dataframe of the pvalues for all samples, with sample_ids in the column headings and probe names in index.
+                # this option will save pvalues for all samples, with sample_ids in the column headings and probe names in index.
                 # this sets poobah to false in kwargs, otherwise some pvalues would be NaN I think.
                 df = consolidate_values_for_sheet(batch_data_containers, postprocess_func_colname='poobah_pval', bit=bit, poobah=False, poobah_sig=poobah_sig, exclude_rs=True)
-                if not batch_size:
-                    pkl_name = 'poobah_values.pkl'
-                else:
-                    pkl_name = f'poobah_values_{batch_num}.pkl'
-                if df.shape[1] > df.shape[0]:
-                    df = df.transpose() # put probes as columns for faster loading.
-                df = df.sort_index().reindex(sorted(df.columns), axis=1)
-                pd.to_pickle(df, Path(data_dir,pkl_name))
-                LOGGER.info(f"saved {pkl_name}")
+                _prepare_save_out_file(df, 'poobah_values')
+
+            if all(['pNegECDF_pval' in e._SampleDataContainer__data_frame.columns for e in batch_data_containers]):
+                # this option will save negative control based pvalues for all samples, with
+                # sample_ids in the column headings and probe names in index.
+                df = consolidate_values_for_sheet(batch_data_containers, postprocess_func_colname='pNegECDF_pval', bit=bit, poobah=False, poobah_sig=poobah_sig, exclude_rs=True)
+                _prepare_save_out_file(df, 'pNegECDF_values')
+
+        # v1.3.0 fixing mem problems: pickling each batch_data_containers object then reloading it later.
 
-        # v1.3.0 fixing mem probs: pickling each batch_data_containers object then reloading it later.
         # consolidating data_containers this will break with really large sample sets, so skip here.
         #if batch_size and batch_size >= 200:
         #    continue
@@ -491,50 +462,29 @@ def run_pipeline(data_dir, array_type=None, export=False, manifest_filepath=None
     del batch_data_containers
 
     if meta_data_frame == True:
-        #sample_sheet.fields is a complete mapping of original and renamed_fields
-        cols = list(sample_sheet.fields.values()) + ['Sample_ID']
-        meta_frame = pd.DataFrame(columns=cols)
-        field_classattr_lookup = {
-        'Sentrix_ID': 'sentrix_id',
-        'Sentrix_Position': 'sentrix_position',
-        'Sample_Group': 'group',
-        'Sample_Name': 'name',
-        'Sample_Plate': 'plate',
-        'Pool_ID': 'pool',
-        'Sample_Well': 'well',
-        'GSM_ID': 'GSM_ID',
-        'Sample_Type': 'type',
-        'Sub_Type': 'sub_type',
-        'Control': 'is_control',
-        }
-        # row contains the renamed fields, and pulls in the original data from sample_sheet
-        for sample in samples:
-            row = {}
-            for field in sample_sheet.fields.keys():
-                if sample_sheet.fields[field] in field_classattr_lookup:
-                    row[ sample_sheet.fields[field] ] = getattr(sample, field_classattr_lookup[sample_sheet.fields[field]] )
-                elif field in sample_sheet.renamed_fields:
-                    row[ sample_sheet.fields[field] ] = getattr(sample, sample_sheet.renamed_fields[field])
-                else:
-                    LOGGER.info(f"extra column: {field} ignored")
-                #    row[ sample_sheet.fields[field] ] = getattr(sample, field)
-            # add the UID that matches m_value/beta value pickles
-            #... unless there's a GSM_ID too
-            # appears that methylprep m_value and beta files only include ID_Position as column names.
-            #if row.get('GSM_ID') != None:
-            #    row['Sample_ID'] = f"{row['GSM_ID']}_{row['Sentrix_ID']}_{row['Sentrix_Position']}"
-            #else:
-            row['Sample_ID'] = f"{row['Sentrix_ID']}_{row['Sentrix_Position']}"
-            meta_frame = meta_frame.append(row, ignore_index=True)
-        meta_frame_filename = f'sample_sheet_meta_data.pkl'
-        meta_frame.to_pickle(Path(data_dir,meta_frame_filename))
+        meta_frame = sample_sheet.build_meta_data(samples)
+        if file_format == 'parquet':
+            meta_frame_filename = f'sample_sheet_meta_data.parquet'
+            meta_frame.to_parquet(Path(data_dir, meta_frame_filename))
+        else:
+            meta_frame_filename = f'sample_sheet_meta_data.pkl'
+            meta_frame.to_pickle(Path(data_dir, meta_frame_filename))
         LOGGER.info(f"saved {meta_frame_filename}")
 
     # FIXED in v1.3.0
     if save_control:
-        control_filename = f'control_probes.pkl'
-        with open(Path(data_dir, control_filename), 'wb') as control_file:
-            pickle.dump(control_snps, control_file)
+        if file_format == 'parquet':
+            control_filename = f'control_probes.parquet'
+            control = pd.concat(control_snps) # creates multiindex
+            (control.reset_index()
+                .rename(columns={'level_0': 'Sentrix_ID', 'level_1': 'IlmnID'})
+                .astype({'IlmnID':str})
+                .to_parquet('control_probes.parquet')
+            )
+        else:
+            control_filename = f'control_probes.pkl'
+            with open(Path(data_dir, control_filename), 'wb') as control_file:
+                pickle.dump(control_snps, control_file)
         LOGGER.info(f"saved {control_filename}")
 
     # summarize any processing errors
@@ -559,12 +509,12 @@ def run_pipeline(data_dir, array_type=None, export=False, manifest_filepath=None
     # consolidate batches and delete parts, if possible
     for file_type in ['beta_values', 'm_values', 'meth_values', 'unmeth_values',
         'noob_meth_values', 'noob_unmeth_values', 'mouse_probes', 'poobah_values']: # control_probes.pkl not included yet
-        test_parts = list([str(temp_file) for temp_file in Path(data_dir).rglob(f'{file_type}*.pkl')])
+        test_parts = list([str(temp_file) for temp_file in Path(data_dir).rglob(f'{file_type}*.{suffix}')])
         num_batches = len(test_parts)
         # ensures that only the file_types that appear to be selected get merged.
         #print(f"DEBUG num_batches {num_batches}, batch_size {batch_size}, file_type {file_type}")
         if batch_size and num_batches >= 1: #--- if the batch size was larger than the number of total samples, this will still drop the _1
-            merge_batches(num_batches, data_dir, file_type)
+            merge_batches(num_batches, data_dir, file_type, file_format)
 
     # reload all the big stuff -- after everything important is done.
     # attempts to consolidate all the batch_files below, if they'll fit in memory.
@@ -611,7 +561,8 @@ class SampleDataContainer(SigSet):
 
     def __init__(self, idat_dataset_pair, manifest=None, retain_uncorrected_probe_intensities=False,
                  bit='float32', pval=False, poobah_decimals=3, poobah_sig=0.05, do_noob=True,
-                 quality_mask=True, switch_probes=True, do_nonlinear_dye_bias=True, debug=False, sesame=True):
+                 quality_mask=True, switch_probes=True, do_nonlinear_dye_bias=True, debug=False, sesame=True,
+                 pneg_ecdf=False, file_format='csv'):
         self.debug = debug
         self.do_noob = do_noob
         self.pval = pval
@@ -625,7 +576,10 @@ class SampleDataContainer(SigSet):
         self.sample = idat_dataset_pair['sample']
         self.retain_uncorrected_probe_intensities=retain_uncorrected_probe_intensities
         self.sesame = sesame # defines offsets in functions
+        # pneg_ecdf defines if negative control based pvalue is calculated - will use poobah_decimals for rounding
+        self.pneg_ecdf = pneg_ecdf
         self.data_type = 'float32' if bit == None else bit # options: (float64, float32, or float16)
+        self.file_format = file_format
         if debug:
             print(f'DEBUG SDC: sesame {self.sesame} switch {self.switch_probes} noob {self.do_noob} poobah {self.pval} mask {self.quality_mask}, dye {self.do_nonlinear_dye_bias}')
 
@@ -683,6 +637,7 @@ class SampleDataContainer(SigSet):
             return self.__data_frame
 
         pval_probes_df = _pval_sesame_preprocess(self) if self.pval == True else None
+        pneg_ecdf_probes_df = _pval_neg_ecdf(self) if self.pneg_ecdf == True else None
         # output: df with one column named 'poobah_pval'
         quality_mask_df = _apply_sesame_quality_mask(self) if self.quality_mask == True else None
         # output: df with one column named 'quality_mask' | if not supported array / custom array: returns nothing.
@@ -736,6 +691,10 @@ class SampleDataContainer(SigSet):
             pval_probes_df = pval_probes_df.loc[ ~pval_probes_df.index.duplicated() ]
             self.__data_frame = self.__data_frame.join(pval_probes_df, how='inner')
 
+        if self.pneg_ecdf == True and isinstance(pneg_ecdf_probes_df, pd.DataFrame):
+            pneg_ecdf_probes_df = pneg_ecdf_probes_df.loc[ ~pneg_ecdf_probes_df.index.duplicated() ]
+            self.__data_frame = self.__data_frame.join(pneg_ecdf_probes_df, how='inner')
+
         self.check_for_probe_loss(f"preprocess_noob sesame={self.sesame} --> {self.methylated.shape} {self.unmethylated.shape}")
 
         if self.quality_mask == True and isinstance(quality_mask_df, pd.DataFrame):
@@ -879,7 +838,10 @@ class SampleDataContainer(SigSet):
         #        else:
         #            num_missing = self.__data_frame['meth'].isna().sum() + self.__data_frame['unmeth'].isna().sum()
         #        self.raw_processing_missing_probe_errors.append((output_path, num_missing))
-        this.to_csv(output_path)
+        if self.file_format == 'parquet':
+            this.to_parquet(output_path)
+        else:
+            this.to_csv(output_path)
 
     def _postprocess(self, input_dataframe, postprocess_func, header, offset=None):
         if offset is not None:
